@@ -47,6 +47,7 @@ WriteIncludes(FILE *file)
     fprintf(file, "#include <mach/mig_support.h>\n");
     if (IsKernelServer)
 	fprintf(file, "#include <ipc/ipc_port.h>\n");
+    fprintf(file, "#include <stdint.h>\n");
     fprintf(file, "\n");
 }
 
@@ -216,12 +217,6 @@ WriteEpilog(FILE *file, const statement_t *stats)
     fprintf(file, "}\n");
     fprintf(file, "\n");
 
-    /*
-     * Then, the <subsystem>_server_routine routine
-     */
-    WriteSubsystemServerRoutine(file, "mig_external");
-    fprintf(file, "\n");
-
     /* symtab */
 
     if (GenSymTab) {
@@ -485,7 +480,7 @@ WriteCheckArgSize(FILE *file, const argument_t *arg)
 		arg->argLongForm ? ".msgtl_header" : "");
     }
 
-    if (btype->itTypeSize % 4 != 0)
+    if (btype->itTypeSize % word_size != 0)
 	fprintf(file, "(");
 
     if (multiplier > 1)
@@ -493,10 +488,10 @@ WriteCheckArgSize(FILE *file, const argument_t *arg)
 
     fprintf(file, "In%dP->%s", arg->argRequestPos, count->argMsgField);
 
-    /* If the base type size of the data field isn`t a multiple of 4,
+    /* If the base type size of the data field isn`t a multiple of word_size,
        we have to round up. */
-    if (btype->itTypeSize % 4 != 0)
-	fprintf(file, " + 3) & ~3");
+    if (btype->itTypeSize % word_size != 0)
+	fprintf(file, " + %d) & ~%d", word_size - 1, word_size - 1);
 
     if (ptype->itIndefinite) {
 	fprintf(file, " : sizeof(%s *)", FetchServerType(btype));
@@ -532,8 +527,8 @@ WriteCheckMsgSize(FILE *file, const argument_t *arg)
 	boolean_t LastVarArg = arg->argRequestPos+1 == rt->rtNumRequestVar;
 
 	/* calculate the actual size in bytes of the data field.  note
-	   that this quantity must be a multiple of four.  hence, if
-	   the base type size isn't a multiple of four, we have to
+	   that this quantity must be a multiple of word_size.  hence, if
+	   the base type size isn't a multiple of word_size, we have to
 	   round up.  note also that btype->itNumber must
 	   divide btype->itTypeSize (see itCalculateSizeInfo). */
 
@@ -755,6 +750,18 @@ WriteServerCallArg(FILE *file, const argument_t *arg)
     const ipc_type_t *it = arg->argType;
     boolean_t NeedClose = FALSE;
 
+    if (IsKernelServer) {
+        /* If the type (incl. array) is handled differently, then we need to
+           cast it to the real argument type.  */
+        if (it->itKernelPort ||
+            it->itInLine && it->itVarArray && it->itElement->itKernelPort) {
+            /* Some arguments are transformed into the correct type already.  */
+            if (!akCheckAll(arg->argKind, akbSendRcv|akbVarNeeded))
+              fprintf(file, "(%s%s)", it->itTransType,
+                      arg->argByReferenceServer ? "*" : "");
+        }
+    }
+
     if (arg->argByReferenceServer)
 	fprintf(file, "&");
 
@@ -1077,7 +1084,7 @@ WriteArgSize(FILE *file, const argument_t *arg)
 		arg->argLongForm ? ".msgtl_header" : "");
     }
 
-    if (bsize % 4 != 0)
+    if (bsize % word_size != 0)
 	fprintf(file, "(");
 
     if (bsize > 1)
@@ -1090,11 +1097,11 @@ WriteArgSize(FILE *file, const argument_t *arg)
 	fprintf(file, "%s", count->argVarName);
 
     /*
-     * If the base type size is not a multiple of sizeof(int) [4],
+     * If the base type size is not a multiple of word_size,
      * we have to round up.
      */
-    if (bsize % 4 != 0)
-	fprintf(file, " + 3) & ~3");
+    if (bsize % word_size != 0)
+	fprintf(file, " + %d) & ~%d", word_size - 1, word_size - 1);
 
     if (ptype->itIndefinite) {
 	fprintf(file, " : sizeof(%s *)",
